package zed;
// @author Spirolone
import java.io.*;
import java.util.zip.CRC32;

public class CreateZed {

    static RandomAccessFile zedFile;
    static byte[] fileHeaderA = new byte[46];
    static byte[] fileHeaderB = new byte[36];

    public static void main(String[] args) {
        try{
            String folderName;
            if (args.length > 0)
                folderName = args[0];
            else
                folderName = "DATA_004";
            File source = new File(folderName);
            // Read Data.bin
            RandomAccessFile data = new RandomAccessFile("Data.bin", "r");
            byte[] filesFromData = new byte[(int)(data.length()-422)];
            data.read(filesFromData);
            byte[] filesHeadersFromData = new byte[340];
            data.read(filesHeadersFromData);
            data.read(fileHeaderA);
            data.read(fileHeaderB);
            data.close();
            // Start writing Zed File
            File destination = new File(folderName+"_Bis.ZED");
            destination.delete();
            zedFile = new RandomAccessFile(destination, "rw");
            zedFile.write(filesFromData);
            // Add all the files from source folder
            byte[] centralDir = addFiles(filesHeadersFromData, source);
            // Xor
            for (int i=filesHeadersFromData.length; i<centralDir.length; i++)
                centralDir[i] ^= centralDir[i-1];
            long centralDirOffset = zedFile.getFilePointer();       // Central Directory Offset
            zedFile.write(centralDir);      // Write Central Directory
            // Write last 8 Bytes
            byte[] tail = new byte[8];
            tail[0] = (byte)(centralDir.length & 0xFF);
            tail[2] = (byte)((centralDir.length>>8) & 0xFF);
            tail[4] = (byte)((centralDir.length>>16) & 0xFF);
            tail[6] = (byte)((centralDir.length>>24) & 0xFF);
            tail[1] = (byte)(centralDirOffset & 0xFF);
            tail[3] = (byte)((centralDirOffset>>8) & 0xFF);
            tail[5] = (byte)((centralDirOffset>>16) & 0xFF);
            tail[7] = (byte)((centralDirOffset>>24) & 0xFF);
            tail[0] ^= tail[7];
            for (int i=1; i<8; i++)
                tail[i] ^= tail[i-1];
            zedFile.write(tail);
            zedFile.close();
        }
        catch (Exception e){
            System.out.printf("Error");
        }
    }
    
    static byte[] addFiles(byte[] centralDir, File source) throws IOException {
        // Create File Header
        String fileName = source.toString();
        fileName = fileName.replace("\\", "/");
        if (source.isDirectory())
            fileName += "/";
        byte[] fileHeader = new byte[82+fileName.length()];
        System.arraycopy(fileHeaderA, 0, fileHeader, 0, 46);
        System.arraycopy(fileHeaderB, 0, fileHeader, fileHeader.length-36, 36);
        System.arraycopy(numToBytes(fileName.length(), 2), 0, fileHeader, 28, 2);      // File Name Len
        System.arraycopy(numToBytes(zedFile.getFilePointer(), 4), 0, fileHeader, 42, 4);        // Offset
        System.arraycopy(fileName.getBytes(), 0, fileHeader, 46, fileName.length());        // File Name
        byte[] newCentralDir = new byte[centralDir.length+fileHeader.length];       // Add space for new File Header in Central Dir
        System.arraycopy(centralDir, 0, newCentralDir, 0, centralDir.length);
        if (source.isDirectory()) {
            System.arraycopy(fileHeader, 0, newCentralDir, centralDir.length, fileHeader.length);
            File filesList[] = source.listFiles();
            for (int i=0; i<filesList.length; i++)
                newCentralDir = addFiles(newCentralDir, filesList[i]);      // Recursive call for every File in folder
        } else {
            // Read File
            RandomAccessFile localFile = new RandomAccessFile(source, "r");
            byte[] fileContent = new byte[(int)localFile.length()];
            localFile.read(fileContent);
            localFile.close();
            CRC32 fileCrc32 = new CRC32();
            fileCrc32.reset();
            fileCrc32.update(fileContent);
            System.arraycopy(numToBytes(fileCrc32.getValue(), 4), 0, fileHeader, 16, 4);        // Crc-32
            System.arraycopy(numToBytes(fileContent.length, 4), 0, fileHeader, 20, 4);      // Compressed Size
            System.arraycopy(numToBytes(fileContent.length, 4), 0, fileHeader, 24, 4);      // Uncompressed Size
            fileHeader[38] = 0x20;        // External Attr.
            System.arraycopy(fileHeader, 0, newCentralDir, centralDir.length, fileHeader.length);
            zedFile.write(fileContent);     // Write Local File in Zed File
        }
        return newCentralDir;
    }
    
    static byte[] numToBytes(long num, int dim) {
        byte [] result = new byte[dim];
        for (int i=0; i<dim; i++)
            result[i] = (byte)((num>>(8*i)) & 0xFF);
        return result;
    }
}